#!/usr/bin/env python
"""DragStrip.py  For timing Hot Wheels or Match Box cars on a track, like a drag strip"""

__author__      = "James Dilts"
__copyright__   = "Copyright 2021"

#from threading import Thread
import time
#from gpiozero.tools import all_values
from gpiozero import LEDBoard
import gpiozero 
from random import *

#Gate Name Constants
TIME_GREEN =0
TIME_START = 1
TIME_30 = 2
TIME_330 = 3
TIME_660 = 4
TIME_1000 = 5
TIME_FINISH = 6

#Initialize time slip
TIME_SLIP = [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]

""" Hot Wheels/Matchbox track laser gate made with Adafruit Break Beam Sensors """
""" For measuring speed of car by using the entry time and exit time when it goes through the gate """
""" Connect Black wires to ground together """
""" Connect Red wires to a GPIO pin to turn 3.3v power on and off as needed """
""" Connect White wire to a GPIO pin for input from the break beam sensor """
class LaserGate():
    # Constants for gpiozero.InputDevice beam detection
    BEAM_DETECTED = 0
    BEAM_BROKEN = 1
    
    def __init__(self, gate_name, power_pin, beam_pin, car_length_16ths):
        self.gate_name = gate_name
        self.gate_entry_time = 0.0
        self.gate_exit_time = 0.0
        self.stop_gate = False
        self.gate_count = 0
        self.car_length_inches = (car_length_16ths / 16)
        # Turn power on to the break beam sensor
        self.power_supply = gpiozero.OutputDevice(power_pin)
        self.power_supply.on()
        # Setup Break Beam Sensor
        self.Gate = gpiozero.InputDevice(beam_pin, pull_up=True, active_state=None, pin_factory=None)
        
    def get_gate_entry_time(self):
        return self.gate_entry_time

    def set_gate_entry_time(self, new_time):
        self.gate_entry_time = new_time
        temp_time = TIME_SLIP[self.gate_name]
        temp_time = [new_time,temp_time[1]]
        TIME_SLIP[self.gate_name] = temp_time

    def get_gate_exit_time(self):
        return self.gate_exit_time
    
    def set_gate_exit_time(self, new_time):
        self.gate_exit_time = new_time
        temp_time = TIME_SLIP[self.gate_name]
        temp_time = [temp_time[0],new_time]
        TIME_SLIP[self.gate_name] = temp_time

    def get_elapsed_time(self):
        if (self.get_gate_exit_time() != 0.0) and (self.get_gate_entry_time() != 0.0):
            return self.get_gate_exit_time() - self.get_gate_entry_time()
        else:
            return -1
        
    def get_actual_speed(self):
        feet_traveled = self.car_length_inches / 12
        local_elapsed_time = self.get_elapsed_time()
        feet_per_second = (1 / local_elapsed_time) * feet_traveled
        return feet_per_second * 0.682
    
    def get_stop_gate(self):
        return self.stop_gate
    
    def set_stop_gate(self):
        self.stop_gate = True
        
    def get_gate_count(self):
        return self.gate_count
        
    def run_gate(self):
        # variable to exit while loop after car exits beam/gate
        self.stop_gate = False
        print("starting gate " + str(self.gate_name))
        while not self.stop_gate: # or Close_Gate):
            if self.Gate.value == self.BEAM_BROKEN:
                print("***   Car Staged   ***")
                self.gate_count += 1
                self.set_gate_entry_time(time.perf_counter())
                while self.Gate.value == self.BEAM_BROKEN:
                    time.sleep(0.0005)
                self.set_gate_exit_time(time.perf_counter())
                self.stop_gate = True
#                Close_Gate = True
            else:
                time.sleep(0.0005)
        print("exit start")
            
# class for starting line gate which only uses the exit time and will be used for upgrades
class StartGate(LaserGate):
    
    def __init__(self, gate_name, power_pin, beam_pin, car_length_16ths):
        self.reaction_time = 0.0
        LaserGate.__init__(self, gate_name, power_pin, beam_pin, car_length_16ths)

    def get_reaction_time(self):
        reaction_time = TIME_SLIP[TIME_GREEN, 0] - self.get_gate_exit_time()
        return reaction_time
    
    def run_gate(self):
        LaserGate.run_gate(self)

        
if __name__ == '__main__':
    CarLength = 1
    
    print("Enter length of track in 2 parts. Feet <ENTER> then Inches <ENTER>")
    TrackFeet = float(input("Enter feet:  "))
    TrackInches = float(input("Enter inches:  "))
    TrackLength = TrackFeet + (TrackInches/12)

    StartingLine = StartGate(TIME_START, 12, 13, CarLength) #, 
    FinishLine = LaserGate(TIME_FINISH, 16, 17, CarLength)

    while (CarLength != 100):
        
        CarLength = 1
        
        while (CarLength < 10) or ((CarLength > 75) and (not (CarLength == 100))):
            CarLength = int(input("Input car length in 16ths between 10 and 75 [100 to quit]."))

        if CarLength == 100:
            pass
        else:
            skip = input("Press <enter> when car is staged.")
            StartingLine.car_length_inches = CarLength / 16
            FinishLine.car_length_inches = CarLength / 16
            
            StartingLine.run_gate()
            FinishLine.run_gate()
            
            print(FinishLine.get_gate_count())
            while FinishLine.get_gate_count() < 1:
                time.sleep(0.5)
            Finish_Slip = TIME_SLIP[TIME_FINISH]
            Start_Slip = TIME_SLIP[TIME_START]
            TrackTime = Finish_Slip[0] - Start_Slip[1]
            AverageSpeed = (((60 * 60) / TrackTime) * TrackLength) / 5280
            ScaleAverageSpeed = AverageSpeed * 64
            FinishLineSpeed = FinishLine.get_actual_speed()
            ScaleFinishLineSpeed = FinishLineSpeed * 64
            
            print("GREAT RACE!")
            print("Finish Time = " + str(round(TrackTime,3)) + " seconds")
            print("Average speed was " + str(round(AverageSpeed,2)) + " mph")
            print("Average scale speed was " + str(round(ScaleAverageSpeed,2)) + " mph")
            print("Finish Line speed was *** " + str(round(FinishLineSpeed,2)) + " MPH ***")
            print("Finish Line scale speed was **********   " + str(round(ScaleFinishLineSpeed,2)) + " MPH!!!   **********")
            
            StartingLine.set_stop_gate()
            FinishLine.set_stop_gate()
    
    

    
